# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import (
    PhoneNumberType,
    PhoneNumberStatus,
    phone_number_list_params,
    phone_number_update_params,
    phone_number_purchase_params,
    phone_number_requirements_params,
    phone_number_search_available_params,
)
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.phone_number_type import PhoneNumberType
from ..types.owned_phone_number import OwnedPhoneNumber
from ..types.phone_number_status import PhoneNumberStatus
from ..types.phone_number_update_response import PhoneNumberUpdateResponse
from ..types.phone_number_purchase_response import PhoneNumberPurchaseResponse
from ..types.phone_number_retrieve_response import PhoneNumberRetrieveResponse
from ..types.phone_number_requirements_response import PhoneNumberRequirementsResponse
from ..types.phone_number_search_available_response import PhoneNumberSearchAvailableResponse

__all__ = ["PhoneNumbersResource", "AsyncPhoneNumbersResource"]


class PhoneNumbersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return PhoneNumbersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Get details of a specific phone number.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._get(
            f"/v1/phone-numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    def update(
        self,
        phone_number_id: str,
        *,
        name: Optional[str] | Omit = omit,
        sender_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Update a phone number's name or sender assignment.

        Args:
          name: Custom name for the phone number. Set to null to clear.

          sender_id: Sender ID to assign the phone number to. Set to null to unassign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._patch(
            f"/v1/phone-numbers/{phone_number_id}",
            body=maybe_transform(
                {
                    "name": name,
                    "sender_id": sender_id,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: PhoneNumberStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[OwnedPhoneNumber]:
        """
        List all phone numbers owned by this project.

        Args:
          cursor: Pagination cursor.

          status: Filter by phone number status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/phone-numbers",
            page=SyncCursor[OwnedPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=OwnedPhoneNumber,
        )

    def purchase(
        self,
        *,
        phone_number: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberPurchaseResponse:
        """Purchase an available phone number.

        The first US phone number is free for each
        team.

        Args:
          phone_number: Phone number in E.164 format.

          name: Optional custom name for the phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/phone-numbers",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "name": name,
                },
                phone_number_purchase_params.PhoneNumberPurchaseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberPurchaseResponse,
        )

    def release(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Release a phone number.

        The phone number must not be assigned to a sender.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/phone-numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def requirements(
        self,
        *,
        country_code: str,
        type: PhoneNumberType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRequirementsResponse:
        """
        Get regulatory requirements for purchasing phone numbers in a specific country.
        Some countries require additional documentation (addresses, identity documents)
        before phone numbers can be activated.

        Args:
          country_code: Two-letter ISO country code.

          type: Type of phone number (local, mobile, tollFree).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/phone-numbers/requirements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country_code": country_code,
                        "type": type,
                    },
                    phone_number_requirements_params.PhoneNumberRequirementsParams,
                ),
            ),
            cast_to=PhoneNumberRequirementsResponse,
        )

    def search_available(
        self,
        *,
        country_code: str,
        contains: str | Omit = omit,
        limit: int | Omit = omit,
        type: PhoneNumberType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberSearchAvailableResponse:
        """
        Search for available phone numbers to purchase by country and type.

        Args:
          country_code: Two-letter ISO country code.

          contains: Search for numbers containing this string.

          limit: Maximum number of results to return.

          type: Type of phone number to search for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/phone-numbers/available",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "country_code": country_code,
                        "contains": contains,
                        "limit": limit,
                        "type": type,
                    },
                    phone_number_search_available_params.PhoneNumberSearchAvailableParams,
                ),
            ),
            cast_to=PhoneNumberSearchAvailableResponse,
        )


class AsyncPhoneNumbersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncPhoneNumbersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRetrieveResponse:
        """
        Get details of a specific phone number.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._get(
            f"/v1/phone-numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberRetrieveResponse,
        )

    async def update(
        self,
        phone_number_id: str,
        *,
        name: Optional[str] | Omit = omit,
        sender_id: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberUpdateResponse:
        """
        Update a phone number's name or sender assignment.

        Args:
          name: Custom name for the phone number. Set to null to clear.

          sender_id: Sender ID to assign the phone number to. Set to null to unassign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._patch(
            f"/v1/phone-numbers/{phone_number_id}",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "sender_id": sender_id,
                },
                phone_number_update_params.PhoneNumberUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberUpdateResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: PhoneNumberStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OwnedPhoneNumber, AsyncCursor[OwnedPhoneNumber]]:
        """
        List all phone numbers owned by this project.

        Args:
          cursor: Pagination cursor.

          status: Filter by phone number status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/phone-numbers",
            page=AsyncCursor[OwnedPhoneNumber],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=OwnedPhoneNumber,
        )

    async def purchase(
        self,
        *,
        phone_number: str,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberPurchaseResponse:
        """Purchase an available phone number.

        The first US phone number is free for each
        team.

        Args:
          phone_number: Phone number in E.164 format.

          name: Optional custom name for the phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/phone-numbers",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "name": name,
                },
                phone_number_purchase_params.PhoneNumberPurchaseParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberPurchaseResponse,
        )

    async def release(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Release a phone number.

        The phone number must not be assigned to a sender.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/phone-numbers/{phone_number_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def requirements(
        self,
        *,
        country_code: str,
        type: PhoneNumberType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberRequirementsResponse:
        """
        Get regulatory requirements for purchasing phone numbers in a specific country.
        Some countries require additional documentation (addresses, identity documents)
        before phone numbers can be activated.

        Args:
          country_code: Two-letter ISO country code.

          type: Type of phone number (local, mobile, tollFree).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/phone-numbers/requirements",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country_code": country_code,
                        "type": type,
                    },
                    phone_number_requirements_params.PhoneNumberRequirementsParams,
                ),
            ),
            cast_to=PhoneNumberRequirementsResponse,
        )

    async def search_available(
        self,
        *,
        country_code: str,
        contains: str | Omit = omit,
        limit: int | Omit = omit,
        type: PhoneNumberType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberSearchAvailableResponse:
        """
        Search for available phone numbers to purchase by country and type.

        Args:
          country_code: Two-letter ISO country code.

          contains: Search for numbers containing this string.

          limit: Maximum number of results to return.

          type: Type of phone number to search for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/phone-numbers/available",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "country_code": country_code,
                        "contains": contains,
                        "limit": limit,
                        "type": type,
                    },
                    phone_number_search_available_params.PhoneNumberSearchAvailableParams,
                ),
            ),
            cast_to=PhoneNumberSearchAvailableResponse,
        )


class PhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.purchase = to_raw_response_wrapper(
            phone_numbers.purchase,
        )
        self.release = to_raw_response_wrapper(
            phone_numbers.release,
        )
        self.requirements = to_raw_response_wrapper(
            phone_numbers.requirements,
        )
        self.search_available = to_raw_response_wrapper(
            phone_numbers.search_available,
        )


class AsyncPhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_raw_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.purchase = async_to_raw_response_wrapper(
            phone_numbers.purchase,
        )
        self.release = async_to_raw_response_wrapper(
            phone_numbers.release,
        )
        self.requirements = async_to_raw_response_wrapper(
            phone_numbers.requirements,
        )
        self.search_available = async_to_raw_response_wrapper(
            phone_numbers.search_available,
        )


class PhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.purchase = to_streamed_response_wrapper(
            phone_numbers.purchase,
        )
        self.release = to_streamed_response_wrapper(
            phone_numbers.release,
        )
        self.requirements = to_streamed_response_wrapper(
            phone_numbers.requirements,
        )
        self.search_available = to_streamed_response_wrapper(
            phone_numbers.search_available,
        )


class AsyncPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.retrieve = async_to_streamed_response_wrapper(
            phone_numbers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            phone_numbers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.purchase = async_to_streamed_response_wrapper(
            phone_numbers.purchase,
        )
        self.release = async_to_streamed_response_wrapper(
            phone_numbers.release,
        )
        self.requirements = async_to_streamed_response_wrapper(
            phone_numbers.requirements,
        )
        self.search_available = async_to_streamed_response_wrapper(
            phone_numbers.search_available,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.senders.agent import tool_list_params, tool_test_params, tool_create_params, tool_update_params
from ....types.senders.agent.agent_tool import AgentTool
from ....types.senders.agent.tool_test_response import ToolTestResponse
from ....types.senders.agent.tool_create_response import ToolCreateResponse
from ....types.senders.agent.tool_update_response import ToolUpdateResponse
from ....types.senders.agent.tool_retrieve_response import ToolRetrieveResponse

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def create(
        self,
        sender_id: str,
        *,
        description: str,
        name: str,
        parameters: tool_create_params.Parameters,
        webhook_url: str,
        enabled: bool | Omit = omit,
        webhook_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolCreateResponse:
        """Create a new tool for an agent.

        Tools allow the agent to call external webhooks.

        Args:
          webhook_url: Must be HTTPS.

          webhook_secret: Optional secret for webhook signature verification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/agent/tools",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "parameters": parameters,
                    "webhook_url": webhook_url,
                    "enabled": enabled,
                    "webhook_secret": webhook_secret,
                },
                tool_create_params.ToolCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolCreateResponse,
        )

    def retrieve(
        self,
        tool_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolRetrieveResponse:
        """
        Get a specific tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._get(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolRetrieveResponse,
        )

    def update(
        self,
        tool_id: str,
        *,
        sender_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        name: str | Omit = omit,
        parameters: tool_update_params.Parameters | Omit = omit,
        webhook_secret: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolUpdateResponse:
        """
        Update a tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._patch(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "name": name,
                    "parameters": parameters,
                    "webhook_secret": webhook_secret,
                    "webhook_url": webhook_url,
                },
                tool_update_params.ToolUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolUpdateResponse,
        )

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        enabled: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[AgentTool]:
        """
        List tools for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/tools",
            page=SyncCursor[AgentTool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "enabled": enabled,
                        "limit": limit,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=AgentTool,
        )

    def delete(
        self,
        tool_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def test(
        self,
        tool_id: str,
        *,
        sender_id: str,
        test_params: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolTestResponse:
        """
        Test a tool by triggering its webhook with test parameters.

        Args:
          test_params: Parameters to pass to the tool for testing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}/test",
            body=maybe_transform({"test_params": test_params}, tool_test_params.ToolTestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolTestResponse,
        )


class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    async def create(
        self,
        sender_id: str,
        *,
        description: str,
        name: str,
        parameters: tool_create_params.Parameters,
        webhook_url: str,
        enabled: bool | Omit = omit,
        webhook_secret: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolCreateResponse:
        """Create a new tool for an agent.

        Tools allow the agent to call external webhooks.

        Args:
          webhook_url: Must be HTTPS.

          webhook_secret: Optional secret for webhook signature verification.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/agent/tools",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "parameters": parameters,
                    "webhook_url": webhook_url,
                    "enabled": enabled,
                    "webhook_secret": webhook_secret,
                },
                tool_create_params.ToolCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolCreateResponse,
        )

    async def retrieve(
        self,
        tool_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolRetrieveResponse:
        """
        Get a specific tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._get(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolRetrieveResponse,
        )

    async def update(
        self,
        tool_id: str,
        *,
        sender_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        name: str | Omit = omit,
        parameters: tool_update_params.Parameters | Omit = omit,
        webhook_secret: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolUpdateResponse:
        """
        Update a tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._patch(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "name": name,
                    "parameters": parameters,
                    "webhook_secret": webhook_secret,
                    "webhook_url": webhook_url,
                },
                tool_update_params.ToolUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolUpdateResponse,
        )

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        enabled: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AgentTool, AsyncCursor[AgentTool]]:
        """
        List tools for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/tools",
            page=AsyncCursor[AgentTool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "enabled": enabled,
                        "limit": limit,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=AgentTool,
        )

    async def delete(
        self,
        tool_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a tool.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def test(
        self,
        tool_id: str,
        *,
        sender_id: str,
        test_params: Dict[str, object],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolTestResponse:
        """
        Test a tool by triggering its webhook with test parameters.

        Args:
          test_params: Parameters to pass to the tool for testing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/agent/tools/{tool_id}/test",
            body=await async_maybe_transform({"test_params": test_params}, tool_test_params.ToolTestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolTestResponse,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.create = to_raw_response_wrapper(
            tools.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tools.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tools.update,
        )
        self.list = to_raw_response_wrapper(
            tools.list,
        )
        self.delete = to_raw_response_wrapper(
            tools.delete,
        )
        self.test = to_raw_response_wrapper(
            tools.test,
        )


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.create = async_to_raw_response_wrapper(
            tools.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tools.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tools.update,
        )
        self.list = async_to_raw_response_wrapper(
            tools.list,
        )
        self.delete = async_to_raw_response_wrapper(
            tools.delete,
        )
        self.test = async_to_raw_response_wrapper(
            tools.test,
        )


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.create = to_streamed_response_wrapper(
            tools.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tools.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tools.update,
        )
        self.list = to_streamed_response_wrapper(
            tools.list,
        )
        self.delete = to_streamed_response_wrapper(
            tools.delete,
        )
        self.test = to_streamed_response_wrapper(
            tools.test,
        )


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.create = async_to_streamed_response_wrapper(
            tools.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tools.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tools.update,
        )
        self.list = async_to_streamed_response_wrapper(
            tools.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            tools.delete,
        )
        self.test = async_to_streamed_response_wrapper(
            tools.test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel
from .agent_knowledge_base import AgentKnowledgeBase

__all__ = ["KnowledgeBaseRetrieveResponse"]


class KnowledgeBaseRetrieveResponse(BaseModel):
    knowledge_base: AgentKnowledgeBase = FieldInfo(alias="knowledgeBase")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["BroadcastContentParam"]


class BroadcastContentParam(TypedDict, total=False):
    """Content for non-text broadcast message types."""

    filename: str
    """Filename for documents."""

    media_id: Annotated[str, PropertyInfo(alias="mediaId")]
    """Media ID if already uploaded."""

    media_url: Annotated[str, PropertyInfo(alias="mediaUrl")]
    """URL of the media file."""

    mime_type: Annotated[str, PropertyInfo(alias="mimeType")]
    """MIME type of the media."""

    template_id: Annotated[str, PropertyInfo(alias="templateId")]
    """Template ID for template messages."""

    template_variables: Annotated[Dict[str, str], PropertyInfo(alias="templateVariables")]
    """Default template variables (can be overridden per contact)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .address import Address
from .._models import BaseModel

__all__ = ["AddressRetrieveResponse"]


class AddressRetrieveResponse(BaseModel):
    address: Address
    """A regulatory address for phone number requirements."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .contact_add_params import ContactAddParams as ContactAddParams
from .contact_list_params import ContactListParams as ContactListParams
from .contact_add_response import ContactAddResponse as ContactAddResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.senders.agent import flow_list_params, flow_create_params, flow_update_params, flow_duplicate_params
from ....types.senders.agent.agent_flow import AgentFlow
from ....types.senders.agent.flow_create_response import FlowCreateResponse
from ....types.senders.agent.flow_update_response import FlowUpdateResponse
from ....types.senders.agent.flow_retrieve_response import FlowRetrieveResponse
from ....types.senders.agent.flow_duplicate_response import FlowDuplicateResponse

__all__ = ["FlowsResource", "AsyncFlowsResource"]


class FlowsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return FlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return FlowsResourceWithStreamingResponse(self)

    def create(
        self,
        sender_id: str,
        *,
        name: str,
        steps: Iterable[flow_create_params.Step],
        trigger: flow_create_params.Trigger,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        priority: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowCreateResponse:
        """
        Create a new flow for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/agent/flows",
            body=maybe_transform(
                {
                    "name": name,
                    "steps": steps,
                    "trigger": trigger,
                    "description": description,
                    "enabled": enabled,
                    "priority": priority,
                },
                flow_create_params.FlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowCreateResponse,
        )

    def retrieve(
        self,
        flow_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowRetrieveResponse:
        """
        Get a specific flow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return self._get(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowRetrieveResponse,
        )

    def update(
        self,
        flow_id: str,
        *,
        sender_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        name: str | Omit = omit,
        priority: int | Omit = omit,
        steps: Iterable[flow_update_params.Step] | Omit = omit,
        trigger: flow_update_params.Trigger | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowUpdateResponse:
        """
        Update a flow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return self._patch(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "name": name,
                    "priority": priority,
                    "steps": steps,
                    "trigger": trigger,
                },
                flow_update_params.FlowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowUpdateResponse,
        )

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        enabled: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[AgentFlow]:
        """
        List flows for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/flows",
            page=SyncCursor[AgentFlow],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "enabled": enabled,
                        "limit": limit,
                    },
                    flow_list_params.FlowListParams,
                ),
            ),
            model=AgentFlow,
        )

    def delete(
        self,
        flow_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a flow.

        Cannot delete flows with active sessions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def duplicate(
        self,
        flow_id: str,
        *,
        sender_id: str,
        new_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowDuplicateResponse:
        """
        Create a copy of an existing flow with a new name.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return self._post(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}/duplicate",
            body=maybe_transform({"new_name": new_name}, flow_duplicate_params.FlowDuplicateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowDuplicateResponse,
        )


class AsyncFlowsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncFlowsResourceWithStreamingResponse(self)

    async def create(
        self,
        sender_id: str,
        *,
        name: str,
        steps: Iterable[flow_create_params.Step],
        trigger: flow_create_params.Trigger,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        priority: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowCreateResponse:
        """
        Create a new flow for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/agent/flows",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "steps": steps,
                    "trigger": trigger,
                    "description": description,
                    "enabled": enabled,
                    "priority": priority,
                },
                flow_create_params.FlowCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowCreateResponse,
        )

    async def retrieve(
        self,
        flow_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowRetrieveResponse:
        """
        Get a specific flow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return await self._get(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowRetrieveResponse,
        )

    async def update(
        self,
        flow_id: str,
        *,
        sender_id: str,
        description: str | Omit = omit,
        enabled: bool | Omit = omit,
        name: str | Omit = omit,
        priority: int | Omit = omit,
        steps: Iterable[flow_update_params.Step] | Omit = omit,
        trigger: flow_update_params.Trigger | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowUpdateResponse:
        """
        Update a flow.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return await self._patch(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "enabled": enabled,
                    "name": name,
                    "priority": priority,
                    "steps": steps,
                    "trigger": trigger,
                },
                flow_update_params.FlowUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowUpdateResponse,
        )

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        enabled: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AgentFlow, AsyncCursor[AgentFlow]]:
        """
        List flows for an agent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/flows",
            page=AsyncCursor[AgentFlow],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "enabled": enabled,
                        "limit": limit,
                    },
                    flow_list_params.FlowListParams,
                ),
            ),
            model=AgentFlow,
        )

    async def delete(
        self,
        flow_id: str,
        *,
        sender_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a flow.

        Cannot delete flows with active sessions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def duplicate(
        self,
        flow_id: str,
        *,
        sender_id: str,
        new_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlowDuplicateResponse:
        """
        Create a copy of an existing flow with a new name.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        if not flow_id:
            raise ValueError(f"Expected a non-empty value for `flow_id` but received {flow_id!r}")
        return await self._post(
            f"/v1/senders/{sender_id}/agent/flows/{flow_id}/duplicate",
            body=await async_maybe_transform({"new_name": new_name}, flow_duplicate_params.FlowDuplicateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=FlowDuplicateResponse,
        )


class FlowsResourceWithRawResponse:
    def __init__(self, flows: FlowsResource) -> None:
        self._flows = flows

        self.create = to_raw_response_wrapper(
            flows.create,
        )
        self.retrieve = to_raw_response_wrapper(
            flows.retrieve,
        )
        self.update = to_raw_response_wrapper(
            flows.update,
        )
        self.list = to_raw_response_wrapper(
            flows.list,
        )
        self.delete = to_raw_response_wrapper(
            flows.delete,
        )
        self.duplicate = to_raw_response_wrapper(
            flows.duplicate,
        )


class AsyncFlowsResourceWithRawResponse:
    def __init__(self, flows: AsyncFlowsResource) -> None:
        self._flows = flows

        self.create = async_to_raw_response_wrapper(
            flows.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            flows.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            flows.update,
        )
        self.list = async_to_raw_response_wrapper(
            flows.list,
        )
        self.delete = async_to_raw_response_wrapper(
            flows.delete,
        )
        self.duplicate = async_to_raw_response_wrapper(
            flows.duplicate,
        )


class FlowsResourceWithStreamingResponse:
    def __init__(self, flows: FlowsResource) -> None:
        self._flows = flows

        self.create = to_streamed_response_wrapper(
            flows.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            flows.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            flows.update,
        )
        self.list = to_streamed_response_wrapper(
            flows.list,
        )
        self.delete = to_streamed_response_wrapper(
            flows.delete,
        )
        self.duplicate = to_streamed_response_wrapper(
            flows.duplicate,
        )


class AsyncFlowsResourceWithStreamingResponse:
    def __init__(self, flows: AsyncFlowsResource) -> None:
        self._flows = flows

        self.create = async_to_streamed_response_wrapper(
            flows.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            flows.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            flows.update,
        )
        self.list = async_to_streamed_response_wrapper(
            flows.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            flows.delete,
        )
        self.duplicate = async_to_streamed_response_wrapper(
            flows.duplicate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .address_status import AddressStatus

__all__ = ["Address"]


class Address(BaseModel):
    """A regulatory address for phone number requirements."""

    id: str

    country_code: str = FieldInfo(alias="countryCode")

    created_at: datetime = FieldInfo(alias="createdAt")

    locality: str

    postal_code: str = FieldInfo(alias="postalCode")

    status: AddressStatus

    street_address: str = FieldInfo(alias="streetAddress")

    administrative_area: Optional[str] = FieldInfo(alias="administrativeArea", default=None)

    business_name: Optional[str] = FieldInfo(alias="businessName", default=None)

    extended_address: Optional[str] = FieldInfo(alias="extendedAddress", default=None)

    first_name: Optional[str] = FieldInfo(alias="firstName", default=None)

    last_name: Optional[str] = FieldInfo(alias="lastName", default=None)

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo
from .agent_provider import AgentProvider

__all__ = ["AgentUpdateParams"]


class AgentUpdateParams(TypedDict, total=False):
    api_key: Annotated[str, PropertyInfo(alias="apiKey")]

    context_window_messages: Annotated[int, PropertyInfo(alias="contextWindowMessages")]

    enabled: bool

    include_contact_metadata: Annotated[bool, PropertyInfo(alias="includeContactMetadata")]

    max_tokens: Annotated[Optional[int], PropertyInfo(alias="maxTokens")]

    model: str

    name: str

    provider: AgentProvider
    """LLM provider for the AI agent."""

    system_prompt: Annotated[str, PropertyInfo(alias="systemPrompt")]

    temperature: Optional[float]

    trigger_on_channels: Annotated[SequenceNotStr[str], PropertyInfo(alias="triggerOnChannels")]

    trigger_on_message_types: Annotated[SequenceNotStr[str], PropertyInfo(alias="triggerOnMessageTypes")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .agent_tool import AgentTool

__all__ = ["ToolUpdateResponse"]


class ToolUpdateResponse(BaseModel):
    tool: AgentTool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RequirementFieldType"]

RequirementFieldType: TypeAlias = Literal["textual", "address", "document", "action"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["AgentDocument"]


class AgentDocument(BaseModel):
    id: str

    chunk_count: int = FieldInfo(alias="chunkCount")
    """Number of chunks created from this document."""

    content_length: int = FieldInfo(alias="contentLength")
    """Length of the document content in characters."""

    created_at: datetime = FieldInfo(alias="createdAt")

    is_processed: bool = FieldInfo(alias="isProcessed")
    """Whether the document has been processed for RAG."""

    knowledge_base_id: str = FieldInfo(alias="knowledgeBaseId")

    title: str

    updated_at: datetime = FieldInfo(alias="updatedAt")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["FlowCreateParams", "Step", "Trigger"]


class FlowCreateParams(TypedDict, total=False):
    name: Required[str]

    steps: Required[Iterable[Step]]

    trigger: Required[Trigger]

    description: str

    enabled: bool

    priority: int


class Step(TypedDict, total=False):
    id: Required[str]
    """Unique step identifier."""

    config: Required[Dict[str, object]]
    """Step configuration (varies by type)."""

    type: Required[Literal["message", "collect", "condition", "tool", "llm", "transfer"]]
    """Type of flow step."""

    next_step_id: Annotated[Optional[str], PropertyInfo(alias="nextStepId")]
    """ID of the next step to execute."""


class Trigger(TypedDict, total=False):
    type: Required[Literal["keyword", "intent", "always", "manual"]]
    """Type of trigger for a flow."""

    intent: str
    """Intent that triggers the flow (for intent type)."""

    keywords: SequenceNotStr[str]
    """Keywords that trigger the flow (for keyword type)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from zavudev import Zavudev, AsyncZavudev
from tests.utils import assert_matches_type
from zavudev.types import (
    Template,
)
from zavudev.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTemplates:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Zavudev) -> None:
        template = client.templates.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Zavudev) -> None:
        template = client.templates.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
            add_security_recommendation=True,
            buttons=[
                {
                    "text": "text",
                    "type": "quick_reply",
                    "otp_type": "COPY_CODE",
                    "package_name": "packageName",
                    "phone_number": "phoneNumber",
                    "signature_hash": "signatureHash",
                    "url": "https://example.com",
                }
            ],
            code_expiration_minutes=1,
            variables=["customer_name", "order_id"],
            whatsapp_category="UTILITY",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Zavudev) -> None:
        response = client.templates.with_raw_response.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Zavudev) -> None:
        with client.templates.with_streaming_response.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Zavudev) -> None:
        template = client.templates.retrieve(
            "templateId",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Zavudev) -> None:
        response = client.templates.with_raw_response.retrieve(
            "templateId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Zavudev) -> None:
        with client.templates.with_streaming_response.retrieve(
            "templateId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            client.templates.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Zavudev) -> None:
        template = client.templates.list()
        assert_matches_type(SyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Zavudev) -> None:
        template = client.templates.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(SyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Zavudev) -> None:
        response = client.templates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = response.parse()
        assert_matches_type(SyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Zavudev) -> None:
        with client.templates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = response.parse()
            assert_matches_type(SyncCursor[Template], template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Zavudev) -> None:
        template = client.templates.delete(
            "templateId",
        )
        assert template is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Zavudev) -> None:
        response = client.templates.with_raw_response.delete(
            "templateId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = response.parse()
        assert template is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Zavudev) -> None:
        with client.templates.with_streaming_response.delete(
            "templateId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = response.parse()
            assert template is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            client.templates.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_submit(self, client: Zavudev) -> None:
        template = client.templates.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_submit_with_all_params(self, client: Zavudev) -> None:
        template = client.templates.submit(
            template_id="templateId",
            sender_id="sender_abc123",
            category="UTILITY",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_submit(self, client: Zavudev) -> None:
        response = client.templates.with_raw_response.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_submit(self, client: Zavudev) -> None:
        with client.templates.with_streaming_response.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_submit(self, client: Zavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            client.templates.with_raw_response.submit(
                template_id="",
                sender_id="sender_abc123",
            )


class TestAsyncTemplates:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
            add_security_recommendation=True,
            buttons=[
                {
                    "text": "text",
                    "type": "quick_reply",
                    "otp_type": "COPY_CODE",
                    "package_name": "packageName",
                    "phone_number": "phoneNumber",
                    "signature_hash": "signatureHash",
                    "url": "https://example.com",
                }
            ],
            code_expiration_minutes=1,
            variables=["customer_name", "order_id"],
            whatsapp_category="UTILITY",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncZavudev) -> None:
        response = await async_client.templates.with_raw_response.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = await response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncZavudev) -> None:
        async with async_client.templates.with_streaming_response.create(
            body="Hi {{1}}, your order {{2}} has been confirmed and will ship within 24 hours.",
            language="en",
            name="order_confirmation",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = await response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.retrieve(
            "templateId",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncZavudev) -> None:
        response = await async_client.templates.with_raw_response.retrieve(
            "templateId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = await response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncZavudev) -> None:
        async with async_client.templates.with_streaming_response.retrieve(
            "templateId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = await response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            await async_client.templates.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.list()
        assert_matches_type(AsyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.list(
            cursor="cursor",
            limit=100,
        )
        assert_matches_type(AsyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncZavudev) -> None:
        response = await async_client.templates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = await response.parse()
        assert_matches_type(AsyncCursor[Template], template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncZavudev) -> None:
        async with async_client.templates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = await response.parse()
            assert_matches_type(AsyncCursor[Template], template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.delete(
            "templateId",
        )
        assert template is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncZavudev) -> None:
        response = await async_client.templates.with_raw_response.delete(
            "templateId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = await response.parse()
        assert template is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncZavudev) -> None:
        async with async_client.templates.with_streaming_response.delete(
            "templateId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = await response.parse()
            assert template is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            await async_client.templates.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_submit(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_submit_with_all_params(self, async_client: AsyncZavudev) -> None:
        template = await async_client.templates.submit(
            template_id="templateId",
            sender_id="sender_abc123",
            category="UTILITY",
        )
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_submit(self, async_client: AsyncZavudev) -> None:
        response = await async_client.templates.with_raw_response.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        template = await response.parse()
        assert_matches_type(Template, template, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_submit(self, async_client: AsyncZavudev) -> None:
        async with async_client.templates.with_streaming_response.submit(
            template_id="templateId",
            sender_id="sender_abc123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            template = await response.parse()
            assert_matches_type(Template, template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_submit(self, async_client: AsyncZavudev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `template_id` but received ''"):
            await async_client.templates.with_raw_response.submit(
                template_id="",
                sender_id="sender_abc123",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ..types import contact_list_params, contact_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.contact import Contact

__all__ = ["ContactsResource", "AsyncContactsResource"]


class ContactsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return ContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return ContactsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        contact_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """
        Get contact

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        return self._get(
            f"/v1/contacts/{contact_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )

    def update(
        self,
        contact_id: str,
        *,
        default_channel: Optional[Literal["sms", "whatsapp", "telegram", "email", "instagram", "voice"]] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """Update contact

        Args:
          default_channel: Preferred channel for this contact.

        Set to null to clear.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        return self._patch(
            f"/v1/contacts/{contact_id}",
            body=maybe_transform(
                {
                    "default_channel": default_channel,
                    "metadata": metadata,
                },
                contact_update_params.ContactUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        phone_number: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[Contact]:
        """
        List contacts with their communication channels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contacts",
            page=SyncCursor[Contact],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "phone_number": phone_number,
                    },
                    contact_list_params.ContactListParams,
                ),
            ),
            model=Contact,
        )

    def retrieve_by_phone(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """
        Get contact by phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/v1/contacts/phone/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )


class AsyncContactsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContactsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncContactsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContactsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncContactsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        contact_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """
        Get contact

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        return await self._get(
            f"/v1/contacts/{contact_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )

    async def update(
        self,
        contact_id: str,
        *,
        default_channel: Optional[Literal["sms", "whatsapp", "telegram", "email", "instagram", "voice"]] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """Update contact

        Args:
          default_channel: Preferred channel for this contact.

        Set to null to clear.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not contact_id:
            raise ValueError(f"Expected a non-empty value for `contact_id` but received {contact_id!r}")
        return await self._patch(
            f"/v1/contacts/{contact_id}",
            body=await async_maybe_transform(
                {
                    "default_channel": default_channel,
                    "metadata": metadata,
                },
                contact_update_params.ContactUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        phone_number: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Contact, AsyncCursor[Contact]]:
        """
        List contacts with their communication channels.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/contacts",
            page=AsyncCursor[Contact],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "phone_number": phone_number,
                    },
                    contact_list_params.ContactListParams,
                ),
            ),
            model=Contact,
        )

    async def retrieve_by_phone(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Contact:
        """
        Get contact by phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/v1/contacts/phone/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Contact,
        )


class ContactsResourceWithRawResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.retrieve = to_raw_response_wrapper(
            contacts.retrieve,
        )
        self.update = to_raw_response_wrapper(
            contacts.update,
        )
        self.list = to_raw_response_wrapper(
            contacts.list,
        )
        self.retrieve_by_phone = to_raw_response_wrapper(
            contacts.retrieve_by_phone,
        )


class AsyncContactsResourceWithRawResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.retrieve = async_to_raw_response_wrapper(
            contacts.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            contacts.update,
        )
        self.list = async_to_raw_response_wrapper(
            contacts.list,
        )
        self.retrieve_by_phone = async_to_raw_response_wrapper(
            contacts.retrieve_by_phone,
        )


class ContactsResourceWithStreamingResponse:
    def __init__(self, contacts: ContactsResource) -> None:
        self._contacts = contacts

        self.retrieve = to_streamed_response_wrapper(
            contacts.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            contacts.update,
        )
        self.list = to_streamed_response_wrapper(
            contacts.list,
        )
        self.retrieve_by_phone = to_streamed_response_wrapper(
            contacts.retrieve_by_phone,
        )


class AsyncContactsResourceWithStreamingResponse:
    def __init__(self, contacts: AsyncContactsResource) -> None:
        self._contacts = contacts

        self.retrieve = async_to_streamed_response_wrapper(
            contacts.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            contacts.update,
        )
        self.list = async_to_streamed_response_wrapper(
            contacts.list,
        )
        self.retrieve_by_phone = async_to_streamed_response_wrapper(
            contacts.retrieve_by_phone,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .whatsapp_business_profile_vertical import WhatsappBusinessProfileVertical

__all__ = ["WhatsappBusinessProfile"]


class WhatsappBusinessProfile(BaseModel):
    """WhatsApp Business profile information."""

    about: Optional[str] = None
    """Short description of the business (max 139 characters)."""

    address: Optional[str] = None
    """Physical address of the business (max 256 characters)."""

    description: Optional[str] = None
    """Extended description of the business (max 512 characters)."""

    email: Optional[str] = None
    """Business email address."""

    profile_picture_url: Optional[str] = FieldInfo(alias="profilePictureUrl", default=None)
    """URL of the business profile picture."""

    vertical: Optional[WhatsappBusinessProfileVertical] = None
    """Business category for WhatsApp Business profile."""

    websites: Optional[List[str]] = None
    """Business website URLs (maximum 2)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["RequirementAcceptanceCriteria"]


class RequirementAcceptanceCriteria(BaseModel):
    """Acceptance criteria for a requirement."""

    allowed_values: Optional[List[str]] = FieldInfo(alias="allowedValues", default=None)

    max_length: Optional[int] = FieldInfo(alias="maxLength", default=None)

    min_length: Optional[int] = FieldInfo(alias="minLength", default=None)

    regex_pattern: Optional[str] = FieldInfo(alias="regexPattern", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["RegulatoryDocument"]


class RegulatoryDocument(BaseModel):
    """A regulatory document for phone number requirements."""

    id: str

    created_at: datetime = FieldInfo(alias="createdAt")

    document_type: Literal[
        "passport",
        "national_id",
        "drivers_license",
        "utility_bill",
        "tax_id",
        "business_registration",
        "proof_of_address",
        "other",
    ] = FieldInfo(alias="documentType")

    name: str

    status: Literal["pending", "uploaded", "verified", "rejected"]

    file_size: Optional[int] = FieldInfo(alias="fileSize", default=None)

    mime_type: Optional[str] = FieldInfo(alias="mimeType", default=None)

    rejection_reason: Optional[str] = FieldInfo(alias="rejectionReason", default=None)

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .channel import Channel
from .message_type import MessageType
from .message_content_param import MessageContentParam

__all__ = ["MessageSendParams"]


class MessageSendParams(TypedDict, total=False):
    to: Required[str]
    """Recipient phone number in E.164 format or email address."""

    channel: Channel
    """Delivery channel.

    Use 'auto' for intelligent routing. If omitted with non-text messageType,
    WhatsApp is used. For email recipients, defaults to 'email'.
    """

    content: MessageContentParam
    """Additional content for non-text message types."""

    fallback_enabled: Annotated[bool, PropertyInfo(alias="fallbackEnabled")]
    """Whether to enable automatic fallback to SMS if WhatsApp fails.

    Defaults to true.
    """

    html_body: Annotated[str, PropertyInfo(alias="htmlBody")]
    """HTML body for email messages.

    If provided, email will be sent as multipart with both text and HTML.
    """

    idempotency_key: Annotated[str, PropertyInfo(alias="idempotencyKey")]
    """Optional idempotency key to avoid duplicate sends."""

    message_type: Annotated[MessageType, PropertyInfo(alias="messageType")]
    """Type of message. Defaults to 'text'."""

    metadata: Dict[str, str]
    """Arbitrary metadata to associate with the message."""

    reply_to: Annotated[str, PropertyInfo(alias="replyTo")]
    """Reply-To email address for email messages."""

    subject: str
    """Email subject line.

    Required when channel is 'email' or recipient is an email address.
    """

    text: str
    """Text body for text messages or caption for media messages."""

    zavu_sender: Annotated[str, PropertyInfo(alias="Zavu-Sender")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .flows import (
    FlowsResource,
    AsyncFlowsResource,
    FlowsResourceWithRawResponse,
    AsyncFlowsResourceWithRawResponse,
    FlowsResourceWithStreamingResponse,
    AsyncFlowsResourceWithStreamingResponse,
)
from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .executions import (
    ExecutionsResource,
    AsyncExecutionsResource,
    ExecutionsResourceWithRawResponse,
    AsyncExecutionsResourceWithRawResponse,
    ExecutionsResourceWithStreamingResponse,
    AsyncExecutionsResourceWithStreamingResponse,
)
from .knowledge_bases import (
    KnowledgeBasesResource,
    AsyncKnowledgeBasesResource,
    KnowledgeBasesResourceWithRawResponse,
    AsyncKnowledgeBasesResourceWithRawResponse,
    KnowledgeBasesResourceWithStreamingResponse,
    AsyncKnowledgeBasesResourceWithStreamingResponse,
)

__all__ = [
    "ExecutionsResource",
    "AsyncExecutionsResource",
    "ExecutionsResourceWithRawResponse",
    "AsyncExecutionsResourceWithRawResponse",
    "ExecutionsResourceWithStreamingResponse",
    "AsyncExecutionsResourceWithStreamingResponse",
    "FlowsResource",
    "AsyncFlowsResource",
    "FlowsResourceWithRawResponse",
    "AsyncFlowsResourceWithRawResponse",
    "FlowsResourceWithStreamingResponse",
    "AsyncFlowsResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "KnowledgeBasesResource",
    "AsyncKnowledgeBasesResource",
    "KnowledgeBasesResourceWithRawResponse",
    "AsyncKnowledgeBasesResourceWithRawResponse",
    "KnowledgeBasesResourceWithStreamingResponse",
    "AsyncKnowledgeBasesResourceWithStreamingResponse",
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.senders import AgentExecutionStatus
from ....types.senders.agent import execution_list_params
from ....types.senders.agent_execution import AgentExecution
from ....types.senders.agent_execution_status import AgentExecutionStatus

__all__ = ["ExecutionsResource", "AsyncExecutionsResource"]


class ExecutionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExecutionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return ExecutionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExecutionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return ExecutionsResourceWithStreamingResponse(self)

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: AgentExecutionStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[AgentExecution]:
        """
        List recent agent executions with pagination.

        Args:
          status: Status of an agent execution.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/executions",
            page=SyncCursor[AgentExecution],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    execution_list_params.ExecutionListParams,
                ),
            ),
            model=AgentExecution,
        )


class AsyncExecutionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExecutionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExecutionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExecutionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncExecutionsResourceWithStreamingResponse(self)

    def list(
        self,
        sender_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        status: AgentExecutionStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AgentExecution, AsyncCursor[AgentExecution]]:
        """
        List recent agent executions with pagination.

        Args:
          status: Status of an agent execution.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not sender_id:
            raise ValueError(f"Expected a non-empty value for `sender_id` but received {sender_id!r}")
        return self._get_api_list(
            f"/v1/senders/{sender_id}/agent/executions",
            page=AsyncCursor[AgentExecution],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    execution_list_params.ExecutionListParams,
                ),
            ),
            model=AgentExecution,
        )


class ExecutionsResourceWithRawResponse:
    def __init__(self, executions: ExecutionsResource) -> None:
        self._executions = executions

        self.list = to_raw_response_wrapper(
            executions.list,
        )


class AsyncExecutionsResourceWithRawResponse:
    def __init__(self, executions: AsyncExecutionsResource) -> None:
        self._executions = executions

        self.list = async_to_raw_response_wrapper(
            executions.list,
        )


class ExecutionsResourceWithStreamingResponse:
    def __init__(self, executions: ExecutionsResource) -> None:
        self._executions = executions

        self.list = to_streamed_response_wrapper(
            executions.list,
        )


class AsyncExecutionsResourceWithStreamingResponse:
    def __init__(self, executions: AsyncExecutionsResource) -> None:
        self._executions = executions

        self.list = async_to_streamed_response_wrapper(
            executions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .broadcast_channel import BroadcastChannel
from .broadcast_message_type import BroadcastMessageType
from .broadcast_content_param import BroadcastContentParam

__all__ = ["BroadcastCreateParams"]


class BroadcastCreateParams(TypedDict, total=False):
    channel: Required[BroadcastChannel]
    """Broadcast delivery channel. Use 'smart' for per-contact intelligent routing."""

    name: Required[str]
    """Name of the broadcast campaign."""

    content: BroadcastContentParam
    """Content for non-text broadcast message types."""

    email_html_body: Annotated[str, PropertyInfo(alias="emailHtmlBody")]
    """HTML body for email broadcasts."""

    email_subject: Annotated[str, PropertyInfo(alias="emailSubject")]
    """Email subject line. Required for email broadcasts."""

    idempotency_key: Annotated[str, PropertyInfo(alias="idempotencyKey")]
    """Idempotency key to prevent duplicate broadcasts."""

    message_type: Annotated[BroadcastMessageType, PropertyInfo(alias="messageType")]
    """Type of message for broadcast."""

    metadata: Dict[str, str]

    scheduled_at: Annotated[Union[str, datetime], PropertyInfo(alias="scheduledAt", format="iso8601")]
    """Schedule the broadcast for future delivery."""

    sender_id: Annotated[str, PropertyInfo(alias="senderId")]
    """Sender profile ID. Uses default sender if omitted."""

    text: str
    """Text content or caption. Supports template variables: {{name}}, {{1}}, etc."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .sender_webhook import SenderWebhook

__all__ = ["Sender", "Whatsapp", "WhatsappPaymentStatus"]


class WhatsappPaymentStatus(BaseModel):
    """Payment configuration status from Meta."""

    can_send_templates: Optional[bool] = FieldInfo(alias="canSendTemplates", default=None)
    """Whether template messages can be sent.

    Requires setupStatus=COMPLETE and methodStatus=VALID.
    """

    method_status: Optional[str] = FieldInfo(alias="methodStatus", default=None)
    """Payment method status (VALID, NONE, etc.)."""

    setup_status: Optional[str] = FieldInfo(alias="setupStatus", default=None)
    """Payment setup status (COMPLETE, NOT_STARTED, etc.)."""


class Whatsapp(BaseModel):
    """WhatsApp Business Account information. Only present if a WABA is connected."""

    display_phone_number: Optional[str] = FieldInfo(alias="displayPhoneNumber", default=None)
    """Display phone number."""

    payment_status: Optional[WhatsappPaymentStatus] = FieldInfo(alias="paymentStatus", default=None)
    """Payment configuration status from Meta."""

    phone_number_id: Optional[str] = FieldInfo(alias="phoneNumberId", default=None)
    """WhatsApp phone number ID from Meta."""


class Sender(BaseModel):
    id: str

    name: str

    phone_number: str = FieldInfo(alias="phoneNumber")
    """Phone number in E.164 format."""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)

    email_receiving_enabled: Optional[bool] = FieldInfo(alias="emailReceivingEnabled", default=None)
    """Whether inbound email receiving is enabled for this sender."""

    is_default: Optional[bool] = FieldInfo(alias="isDefault", default=None)
    """Whether this sender is the project's default."""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)

    webhook: Optional[SenderWebhook] = None
    """Webhook configuration for the sender."""

    whatsapp: Optional[Whatsapp] = None
    """WhatsApp Business Account information. Only present if a WABA is connected."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["AddressCreateParams"]


class AddressCreateParams(TypedDict, total=False):
    country_code: Required[Annotated[str, PropertyInfo(alias="countryCode")]]

    locality: Required[str]

    postal_code: Required[Annotated[str, PropertyInfo(alias="postalCode")]]

    street_address: Required[Annotated[str, PropertyInfo(alias="streetAddress")]]

    administrative_area: Annotated[str, PropertyInfo(alias="administrativeArea")]

    business_name: Annotated[str, PropertyInfo(alias="businessName")]

    extended_address: Annotated[str, PropertyInfo(alias="extendedAddress")]

    first_name: Annotated[str, PropertyInfo(alias="firstName")]

    last_name: Annotated[str, PropertyInfo(alias="lastName")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .channel import Channel
from .._models import BaseModel
from .message_type import MessageType
from .message_status import MessageStatus
from .message_content import MessageContent

__all__ = ["Message"]


class Message(BaseModel):
    id: str

    channel: Channel
    """Delivery channel. Use 'auto' for intelligent routing."""

    created_at: datetime = FieldInfo(alias="createdAt")

    message_type: MessageType = FieldInfo(alias="messageType")
    """Type of message.

    Non-text types are supported by WhatsApp and Telegram (varies by type).
    """

    status: MessageStatus

    to: str

    content: Optional[MessageContent] = None
    """Content for non-text message types (WhatsApp and Telegram)."""

    cost: Optional[float] = None
    """MAU cost in USD (charged for first contact of the month)."""

    cost_provider: Optional[float] = FieldInfo(alias="costProvider", default=None)
    """Provider cost in USD (Telnyx, SES, etc.)."""

    cost_total: Optional[float] = FieldInfo(alias="costTotal", default=None)
    """Total cost in USD (MAU + provider cost)."""

    error_code: Optional[str] = FieldInfo(alias="errorCode", default=None)

    error_message: Optional[str] = FieldInfo(alias="errorMessage", default=None)

    from_: Optional[str] = FieldInfo(alias="from", default=None)

    metadata: Optional[Dict[str, str]] = None

    provider_message_id: Optional[str] = FieldInfo(alias="providerMessageId", default=None)
    """Message ID from the delivery provider."""

    sender_id: Optional[str] = FieldInfo(alias="senderId", default=None)

    text: Optional[str] = None
    """Text content or caption."""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["OwnedPhoneNumberPricing"]


class OwnedPhoneNumberPricing(BaseModel):
    is_free_number: Optional[bool] = FieldInfo(alias="isFreeNumber", default=None)
    """Whether this is a free number."""

    monthly_cost: Optional[float] = FieldInfo(alias="monthlyCost", default=None)
    """Monthly cost in cents."""

    monthly_price: Optional[float] = FieldInfo(alias="monthlyPrice", default=None)
    """Monthly price in USD."""

    upfront_cost: Optional[float] = FieldInfo(alias="upfrontCost", default=None)
    """One-time purchase cost in cents."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import regulatory_document_list_params, regulatory_document_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.regulatory_document import RegulatoryDocument
from ..types.regulatory_document_create_response import RegulatoryDocumentCreateResponse
from ..types.regulatory_document_retrieve_response import RegulatoryDocumentRetrieveResponse
from ..types.regulatory_document_upload_url_response import RegulatoryDocumentUploadURLResponse

__all__ = ["RegulatoryDocumentsResource", "AsyncRegulatoryDocumentsResource"]


class RegulatoryDocumentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RegulatoryDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return RegulatoryDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegulatoryDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return RegulatoryDocumentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        document_type: Literal[
            "passport",
            "national_id",
            "drivers_license",
            "utility_bill",
            "tax_id",
            "business_registration",
            "proof_of_address",
            "other",
        ],
        file_size: int,
        mime_type: str,
        name: str,
        storage_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentCreateResponse:
        """Create a regulatory document record after uploading the file.

        Use the upload-url
        endpoint first to get an upload URL.

        Args:
          storage_id: Storage ID from the upload-url endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/documents",
            body=maybe_transform(
                {
                    "document_type": document_type,
                    "file_size": file_size,
                    "mime_type": mime_type,
                    "name": name,
                    "storage_id": storage_id,
                },
                regulatory_document_create_params.RegulatoryDocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentCreateResponse,
        )

    def retrieve(
        self,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentRetrieveResponse:
        """
        Get a specific regulatory document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return self._get(
            f"/v1/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentRetrieveResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[RegulatoryDocument]:
        """
        List regulatory documents for this project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/documents",
            page=SyncCursor[RegulatoryDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    regulatory_document_list_params.RegulatoryDocumentListParams,
                ),
            ),
            model=RegulatoryDocument,
        )

    def delete(
        self,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a regulatory document.

        Cannot delete verified documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def upload_url(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentUploadURLResponse:
        """Get a presigned URL to upload a document file.

        After uploading, use the
        storageId to create the document record.
        """
        return self._post(
            "/v1/documents/upload-url",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentUploadURLResponse,
        )


class AsyncRegulatoryDocumentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRegulatoryDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/zavudev/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegulatoryDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegulatoryDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/zavudev/sdk-python#with_streaming_response
        """
        return AsyncRegulatoryDocumentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        document_type: Literal[
            "passport",
            "national_id",
            "drivers_license",
            "utility_bill",
            "tax_id",
            "business_registration",
            "proof_of_address",
            "other",
        ],
        file_size: int,
        mime_type: str,
        name: str,
        storage_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentCreateResponse:
        """Create a regulatory document record after uploading the file.

        Use the upload-url
        endpoint first to get an upload URL.

        Args:
          storage_id: Storage ID from the upload-url endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/documents",
            body=await async_maybe_transform(
                {
                    "document_type": document_type,
                    "file_size": file_size,
                    "mime_type": mime_type,
                    "name": name,
                    "storage_id": storage_id,
                },
                regulatory_document_create_params.RegulatoryDocumentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentCreateResponse,
        )

    async def retrieve(
        self,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentRetrieveResponse:
        """
        Get a specific regulatory document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return await self._get(
            f"/v1/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentRetrieveResponse,
        )

    def list(
        self,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[RegulatoryDocument, AsyncCursor[RegulatoryDocument]]:
        """
        List regulatory documents for this project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/documents",
            page=AsyncCursor[RegulatoryDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    regulatory_document_list_params.RegulatoryDocumentListParams,
                ),
            ),
            model=RegulatoryDocument,
        )

    async def delete(
        self,
        document_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Delete a regulatory document.

        Cannot delete verified documents.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def upload_url(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegulatoryDocumentUploadURLResponse:
        """Get a presigned URL to upload a document file.

        After uploading, use the
        storageId to create the document record.
        """
        return await self._post(
            "/v1/documents/upload-url",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegulatoryDocumentUploadURLResponse,
        )


class RegulatoryDocumentsResourceWithRawResponse:
    def __init__(self, regulatory_documents: RegulatoryDocumentsResource) -> None:
        self._regulatory_documents = regulatory_documents

        self.create = to_raw_response_wrapper(
            regulatory_documents.create,
        )
        self.retrieve = to_raw_response_wrapper(
            regulatory_documents.retrieve,
        )
        self.list = to_raw_response_wrapper(
            regulatory_documents.list,
        )
        self.delete = to_raw_response_wrapper(
            regulatory_documents.delete,
        )
        self.upload_url = to_raw_response_wrapper(
            regulatory_documents.upload_url,
        )


class AsyncRegulatoryDocumentsResourceWithRawResponse:
    def __init__(self, regulatory_documents: AsyncRegulatoryDocumentsResource) -> None:
        self._regulatory_documents = regulatory_documents

        self.create = async_to_raw_response_wrapper(
            regulatory_documents.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            regulatory_documents.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            regulatory_documents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            regulatory_documents.delete,
        )
        self.upload_url = async_to_raw_response_wrapper(
            regulatory_documents.upload_url,
        )


class RegulatoryDocumentsResourceWithStreamingResponse:
    def __init__(self, regulatory_documents: RegulatoryDocumentsResource) -> None:
        self._regulatory_documents = regulatory_documents

        self.create = to_streamed_response_wrapper(
            regulatory_documents.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            regulatory_documents.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            regulatory_documents.list,
        )
        self.delete = to_streamed_response_wrapper(
            regulatory_documents.delete,
        )
        self.upload_url = to_streamed_response_wrapper(
            regulatory_documents.upload_url,
        )


class AsyncRegulatoryDocumentsResourceWithStreamingResponse:
    def __init__(self, regulatory_documents: AsyncRegulatoryDocumentsResource) -> None:
        self._regulatory_documents = regulatory_documents

        self.create = async_to_streamed_response_wrapper(
            regulatory_documents.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            regulatory_documents.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            regulatory_documents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            regulatory_documents.delete,
        )
        self.upload_url = async_to_streamed_response_wrapper(
            regulatory_documents.upload_url,
        )
